# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

""" ContactListPage emulator for Addressbook App tests """

import logging
from time import sleep

from autopilot.introspection.dbus import StateNotFoundError
from autopilot.matchers import Eventually
from testtools.matchers import GreaterThan
from ubuntuuitoolkit import emulators as uitk

from address_book_app.emulators import get_nearest_parent_matching_type

LOGGER = logging.getLogger(__name__)


class ContactListPage(uitk.UbuntuUIToolkitEmulatorBase):
    """ ContactListPage emulator class """

    def __init__(self, *args):
        self.contacts = None
        self.selection_marks = []
        self.selected_marks = []
        super(ContactListPage, self).__init__(*args)

    def get_contacts(self):
        """
        Returns a list of ContactDelegate objects and populate
        self.selection_marks
        """
        sleep(1)
        self.contacts = self.select_many("ContactDelegate")
        self.selection_marks = []
        for contact in self.contacts:
            if contact.visible:
                mark = contact.select_single("QQuickRectangle",
                                             objectName="selectionMark")
            self.selection_marks.append(mark)
        return self.contacts

    def select_contacts_by_index(self, indices):
        """ Select contacts corresponding to the list of index in indices

        :param indices: List of integers
        """
        self.deselect_all()

        # Select matching indices
        for idx in indices:
            self.selected_marks.append(self.selection_marks[idx])
            self.pointing_device.click_object(self.selection_marks[idx])

    def deselect_all(self):
        """Deselect every contacts"""
        contacts = self.select_many("ContactDelegate")
        self.selected_marks = []
        for contact in contacts:
            if contact.selected:
                mark = contact.select_single("QQuickRectangle",
                                             objectName="selectionMark")
                self.pointing_device.click_object(mark)

    def click_button(self, objectname):
        """Press a button that matches objectname

        :param objectname: Name of the object
        """
        try:
            buttons = self.select_many("Button",
                                       objectName=objectname)
            for button in buttons:
                if button.visible:
                    self.pointing_device.click_object(button)
        except StateNotFoundError:
            LOGGER.error(
                'Button with objectName "{0}" not found.'.format(objectname)
            )
            raise

    def cancel(self):
        """Press the cancel button displayed when pick mode is enabled"""
        self.click_button("DialogButtons.rejectButton")

    def delete(self):
        """Press the delete button displayed when pick mode is enabled"""
        self.click_button("DialogButtons.acceptButton")
        self.get_contacts()

    def get_contact_by_name(self,
                            contact_name,
                            parent_delegate_type='ContactDelegate'):
        """Find a label with text matching contact name.

        :param contact_name: Name of the contact, e.g. 'Fulano de Tal'
        :param parent_delegate_type: 'ContactDelegate' or 'FavoriteDelegate'
        :raises StateNotFoundError: if contact_name is not found
        """
        try:
            assert(lambda: len(self.select_many("Label",
                                                text=contact_name)),
                   Eventually(GreaterThan(0)))
            contact_name_labels = self.select_many(
                "Label",
                text=contact_name)
            for contact_name_label in contact_name_labels:
                # we could have a contact or a favorite
                if get_nearest_parent_matching_type(
                        contact_name_label,
                        parent_delegate_type):
                    return contact_name_label
            raise StateNotFoundError('Label')
        except StateNotFoundError:
            LOGGER.error("Contact {} not found.".format(contact_name))
            raise

    def get_favorite_by_name(self, contact_name):
        """Find a label with text matching contact name under Favorites

        :param contact_name: Name of the contact, e.g. 'Fulano de Tal'
        :raises StateNotFoundError: if contact_name is not found
        """
        return self.get_contact_by_name(
            contact_name,
            parent_delegate_type='FavoriteDelegate')

    def click_contact_by_name(self,
                              contact_name,
                              parent_delegate_type='ContactDelegate'):
        """Click a contact with label matching the given contact name.

        :param contact_name: Name of a contact, e.g. 'Fulano de Tal'
        :param parent_delegate_type: 'ContactDelegate' or 'FavoriteDelegate'
        :raises StateNotFoundError: if contact_name is not found
        """
        contact_name_label = self.get_contact_by_name(
            contact_name, parent_delegate_type)
        self.pointing_device.click_object(contact_name_label)

    def click_favorite_by_name(self,
                               contact_name):
        """Click a favorite with label matching the given contact name.

        :param contact_name: Name of a contact, e.g. 'Fulano de Tal'
        :raises StateNotFoundError: if contact_name is not found
        """
        contact_name_label = self.get_favorite_by_name(
            contact_name)
        self.pointing_device.click_object(contact_name_label)

    def click_phone_number(self, phone_number):
        """Click a label matching the given phone number

        :param phone_number: A phone number, e.g. '3321 1232'
        :raises StateNotFoundError: if phone_number is not found
        """
        # TODO select_many and raise if > 1?
        phone_number_label = self.wait_select_single('Label',
                                                     text=phone_number)
        self.pointing_device.click_object(phone_number_label)
