# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

""" ContactView emulator for Addressbook App tests """

import logging

from autopilot.introspection.dbus import StateNotFoundError
from ubuntuuitoolkit import emulators as uitk

from address_book_app.emulators import get_nearest_parent_matching_type
from address_book_app.emulators.contact_list_page import ContactListPage

LOGGER = logging.getLogger(__name__)


class ContactView(uitk.UbuntuUIToolkitEmulatorBase):
    """ContactView emulator class"""

    def __init__(self, *args):
        super(ContactView, self).__init__(*args)

    def click_button(self, objectname):
        """Press a button that matches objectname

        :param objectname: Name of the object
        :raises StateNotFoundError: if the matching object is not found
        """
        try:
            buttons = self.select_many("Button",
                                       objectName=objectname)
            for button in buttons:
                if button.visible:
                    self.pointing_device.click_object(button)
        except StateNotFoundError:
            LOGGER.error(
                'Button with objectName "{0}" not found.'.format(objectname)
            )
            raise

    def cancel(self):
        """Press the cancel button displayed when pick mode is enabled"""
        self.click_button("DialogButtons.rejectButton")

    def delete(self):
        """Press the delete button displayed when pick mode is enabled"""
        self.click_button("DialogButtons.acceptButton")
        self.get_contacts()

    def get_contact_detail_phone_number_view(self, phone_number):
        """Returns the View associated with the given phone number

        :param phone_number: number listed for contact, e.g. '8675309'
        :raises StateNotFoundError: if phone_number is not found
        """
        phone_number_label = self.select_single('Label',
                                                text=phone_number)
        return get_nearest_parent_matching_type(
            phone_number_label,
            'ContactDetailPhoneNumberView')

    def star_is_illuminated_for_phone_number(self, phone_number):
        """Is the star associated with the given phone number illuminated?

        :param phone_number: number listed for contact, e.g. '8675309'
        :raises StateNotFoundError: if phone_number is not found
        """
        contact_detail = self.get_contact_detail_phone_number_view(
            phone_number)
        return contact_detail.iconSource == 'artwork:/favorite-selected.svg'

    def click_phone_number(self, phone_number):
        """Tap/click the label with text matching the given phone number

        :param phone_number: number listed for contact, e.g. '8675309'
        :raises StateNotFoundError: if phone_number is not found
        """
        phone_number_label = self.wait_select_single('Label',
                                                     text=phone_number)
        self.pointing_device.click_object(phone_number_label)
